@@ -4,7 +4,6 @@ import logging
 
 import ailment
 
-from ... import AnalysesHub
 from .optimization_pass import OptimizationPass, OptimizationPassStage
 
 
@@ -19,12 +18,14 @@ def s2u(s, bits):
 
 class StackCanarySimplifier(OptimizationPass):
     """
-    Removes stack canary checks from AIL graphs.
+    Removes stack canary checks from decompilation results.
     """
 
     ARCHES = ["X86", "AMD64", ] # TODO: fs is x86 only. Figure out how stack canary is loaded in other architectures
     PLATFORMS = ["cgc", "linux"]
     STAGE = OptimizationPassStage.AFTER_GLOBAL_SIMPLIFICATION
+    NAME = "Simplify stack canaries"
+    DESCRIPTION = __doc__.strip()
 
     def __init__(self, func, **kwargs):
 
@@ -260,6 +261,3 @@ class StackCanarySimplifier(OptimizationPass):
                 and expr.addr.operands[0].value == 0x28
                 and isinstance(expr.addr.operands[1], ailment.Expr.Register)
                 and expr.addr.operands[1].reg_offset == fs_reg_offset)
-
-
-AnalysesHub.register_default("StackCanarySimplifier", StackCanarySimplifier)
