@@ -1,6 +1,6 @@
 # pylint:disable=missing-class-docstring,unused-argument
 from collections import defaultdict
-from typing import Optional, Any, Dict, Set, Tuple, DefaultDict, TYPE_CHECKING
+from typing import Optional, Any, Dict, Set, Tuple, Iterable, DefaultDict, TYPE_CHECKING
 
 import ailment
 from ailment import Expression, Block
@@ -32,6 +32,18 @@ class StatementLocation(LocationBase):
     def __repr__(self):
         return f"Loc: Statement@{self.block_addr:x}.{self.block_idx}-{self.stmt_idx}"
 
+    def __hash__(self):
+        return hash((StatementLocation, self.block_addr, self.block_idx, self.stmt_idx))
+
+    def __eq__(self, other):
+        return isinstance(other, StatementLocation) and \
+            self.block_addr == other.block_addr and \
+            self.block_idx == other.block_idx and \
+            self.stmt_idx == other.stmt_idx
+
+    def copy(self):
+        return StatementLocation(self.block_addr, self.block_idx, self.stmt_idx)
+
 
 class ExpressionLocation(LocationBase):
 
@@ -46,6 +58,19 @@ class ExpressionLocation(LocationBase):
     def __repr__(self):
         return f"Loc: Expression@{self.block_addr:x}.{self.block_idx}-{self.stmt_idx}[{self.expr_idx}]"
 
+    def statement_location(self) -> StatementLocation:
+        return StatementLocation(self.block_addr, self.block_idx, self.stmt_idx)
+
+    def __hash__(self):
+        return hash((ExpressionLocation, self.block_addr, self.block_idx, self.stmt_idx, self.expr_idx))
+
+    def __eq__(self, other):
+        return isinstance(other, ExpressionLocation) and \
+                self.block_addr == other.block_addr and \
+                self.block_idx == other.block_idx and \
+                self.stmt_idx == other.stmt_idx and \
+                self.expr_idx == other.expr_idx
+
 
 class ConditionLocation(LocationBase):
 
@@ -58,6 +83,14 @@ class ConditionLocation(LocationBase):
     def __repr__(self):
         return f"Loc: ConditionNode@{self.node_addr:x}.{self.case_idx}"
 
+    def __hash__(self):
+        return hash((ConditionLocation, self.node_addr, self.case_idx))
+
+    def __eq__(self, other):
+        return isinstance(other, ConditionLocation) and \
+                self.node_addr == other.node_addr and \
+                self.case_idx == other.case_idx
+
 
 class ConditionalBreakLocation(LocationBase):
 
@@ -69,13 +102,22 @@ class ConditionalBreakLocation(LocationBase):
     def __repr__(self):
         return f"Loc: ConditionalBreakNode@{self.node_addr:x}"
 
+    def __hash__(self):
+        return hash((ConditionalBreakLocation, self.node_addr))
+
+    def __eq__(self, other):
+        return isinstance(other, ConditionalBreakLocation) and \
+                self.node_addr == other.node_addr
+
 
 class ExpressionUseFinder(AILBlockWalker):
     """
     Find where each variable is used.
 
-    Additionally, determine if the expression being walked is unlikely to be safely folded. For example, we cannot
-    safely fold variable assignments that use Load() because the loaded expression may get updated later by a Store.
+    Additionally, determine if the expression being walked has load expressions inside. Such expressions can only be
+    safely folded if there are no Store statements between the expression defining location and its use sites. For
+    example, we can only safely fold variable assignments that use Load() when there are no Store()s between the
+    assignment and its use site. Otherwise, the loaded expression may get updated later by a Store() statement.
 
     Here is a real AIL block:
 
@@ -89,10 +131,12 @@ class ExpressionUseFinder(AILBlockWalker):
     the second statement.
     """
 
+    __slots__ = ('uses', 'has_load', )
+
     def __init__(self):
         super().__init__()
         self.uses = defaultdict(set)
-        self.unsupported = False
+        self.has_load = False
 
     def _handle_expr(self, expr_idx: int, expr: Expression, stmt_idx: int, stmt: Optional[Statement],
                      block: Optional[Block]) -> Any:
@@ -107,7 +151,7 @@ class ExpressionUseFinder(AILBlockWalker):
 
     def _handle_Load(self, expr_idx: int, expr: ailment.Expr.Load, stmt_idx: int, stmt: Statement,
                      block: Optional[Block]):
-        self.unsupported = True
+        self.has_load = True
         return super()._handle_Load(expr_idx, expr, stmt_idx, stmt, block)
 
 
@@ -150,14 +194,11 @@ class ExpressionCounter(SequenceWalker):
                         # dependency
                         dependency_finder = ExpressionUseFinder()
                         dependency_finder.walk_expression(stmt.src)
-                        if not dependency_finder.unsupported:
-                            dependencies = tuple(set(self._u(v) for v in dependency_finder.uses))
-                        else:
-                            dependencies = ()
+                        dependencies = tuple(set(self._u(v) for v in dependency_finder.uses))
                         self.assignments[u].add((stmt.src,
-                                                 tuple(dependencies),
+                                                 dependencies,
                                                  StatementLocation(node.addr, node.idx, idx),
-                                                 not dependency_finder.unsupported))
+                                                 dependency_finder.has_load))
             if (isinstance(stmt, ailment.Stmt.Call)
                     and isinstance(stmt.ret_expr, ailment.Expr.Register)
                     and stmt.ret_expr.variable is not None):
@@ -165,14 +206,11 @@ class ExpressionCounter(SequenceWalker):
                 if u is not None:
                     dependency_finder = ExpressionUseFinder()
                     dependency_finder.walk_expression(stmt)
-                    if not dependency_finder.unsupported:
-                        dependencies = tuple(set(self._u(v) for v in dependency_finder.uses))
-                    else:
-                        dependencies = ()
+                    dependencies = tuple(set(self._u(v) for v in dependency_finder.uses))
                     self.assignments[u].add((stmt,
-                                             tuple(dependencies),
+                                             dependencies,
                                              StatementLocation(node.addr, node.idx, idx),
-                                             not dependency_finder.unsupported))
+                                             dependency_finder.has_load))
 
         # walk the block and find uses of variables
         use_finder = ExpressionUseFinder()
@@ -357,3 +395,70 @@ class ExpressionFolder(SequenceWalker):
                 node.condition = r
 
         return super()._handle_Loop(node, **kwargs)
+
+
+class StoreStatementFinder(SequenceWalker):
+    """
+    Determine if there are any Store statements between two given statements.
+    """
+    def __init__(self, node, intervals: Iterable[Tuple[StatementLocation,LocationBase]]):
+        handlers = {
+            ConditionNode: self._handle_Condition,
+            CascadingConditionNode: self._handle_CascadingCondition,
+            ConditionalBreakNode: self._handle_ConditionalBreak,
+            ailment.Block: self._handle_Block,
+        }
+
+        self._intervals = intervals
+
+        self._start_to_ends: DefaultDict[StatementLocation,Set[LocationBase]] = defaultdict(set)
+        self._end_to_starts: DefaultDict[LocationBase,Set[StatementLocation]] = defaultdict(set)
+        self.interval_to_hasstore: Dict[Tuple[StatementLocation,StatementLocation],bool] = { }
+        for start, end in intervals:
+            self._start_to_ends[start].add(end)
+            self._end_to_starts[end].add(start)
+
+        self._active_intervals = set()
+
+        super().__init__(handlers)
+        self.walk(node)
+
+    def _handle_Block(self, node: ailment.Block, **kwargs):
+        stmt_loc = StatementLocation(node.addr, node.idx, None)
+        for idx, stmt in enumerate(node.statements):
+            stmt_loc.stmt_idx = idx
+            if stmt_loc in self._start_to_ends:
+                for end in self._start_to_ends[stmt_loc]:
+                    self._active_intervals.add((stmt_loc.copy(), end))
+            if stmt_loc in self._end_to_starts:
+                for start in self._end_to_starts[stmt_loc]:
+                    self._active_intervals.discard((start, stmt_loc))
+            if isinstance(stmt, ailment.Stmt.Store):
+                for interval in self._active_intervals:
+                    self.interval_to_hasstore[interval] = True
+
+    def _handle_Condition(self, node, **kwargs):
+        cond_loc = ConditionLocation(node.addr)
+        if cond_loc in self._end_to_starts:
+            for start in self._end_to_starts[cond_loc]:
+                self._active_intervals.discard((start, cond_loc))
+        super()._handle_Condition(node, **kwargs)
+
+    def _handle_CascadingCondition(self, node: CascadingConditionNode, **kwargs):
+        cond_loc = ConditionLocation(node.addr, None)
+        for idx in range(len(node.condition_and_nodes)):
+            cond_loc.case_idx = idx
+            if cond_loc in self._end_to_starts[cond_loc]:
+                for start in self._end_to_starts[cond_loc]:
+                    self._active_intervals.discard((start, cond_loc))
+        super()._handle_CascadingCondition(node, **kwargs)
+
+    def _handle_ConditionalBreak(self, node: ConditionalBreakNode, **kwargs):
+        cond_break_loc = ConditionalBreakLocation(node.addr)
+        if cond_break_loc in self._end_to_starts:
+            for start in self._end_to_starts[cond_break_loc]:
+                self._active_intervals.discard((start, cond_break_loc))
+        super()._handle_ConditionalBreak(node, **kwargs)
+
+    def has_store(self, start: StatementLocation, end: StatementLocation) -> bool:
+        return self.interval_to_hasstore.get((start, end), False)
