@@ -1,5 +1,6 @@
 import threading
 
+
 class PicklableLock:
     """
     Normal thread-locks are not pickleable. This provides a pickleable lock by mandating that the lock is unlocked
@@ -8,7 +9,7 @@ class PicklableLock:
     _LOCK = threading.Lock
 
     def __init__(self, *args, **kwargs):
-        self._lock = self._LOCK(*args, **kwargs)  # pylint: disable=too-many-function-args
+        self._lock = self.__class__._LOCK(*args, **kwargs)  # pylint: disable=too-many-function-args
 
     def __enter__(self):
         return self._lock.__enter__()
@@ -30,6 +31,7 @@ class PicklableLock:
             raise TypeError("Why are you pickling a locked lock")
         return type(self), ()
 
+
 class PicklableRLock(PicklableLock):
     """
     Same as above, but uses RLock instead of Lock for locking. Note that RLock does not provide an interface to tell
