@@ -13,11 +13,11 @@ _l = logging.getLogger(name=__name__)
 lib = SimLibrary()
 lib.set_default_cc('X86', SimCCStdcall)
 lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
-lib.set_library_names("api-ms-win-security-isolatedcontainer-l1-1-1.dll")
+lib.set_library_names("api-ms-win-security-isolatedcontainer-l1-1-0.dll")
 prototypes = \
     {
         # 
-        'IsProcessInWDAGContainer': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Reserved", "isProcessInWDAGContainer"]),
+        'IsProcessInIsolatedContainer': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["isProcessInIsolatedContainer"]),
     }
 
 lib.set_prototypes(prototypes)
