@@ -30,7 +30,7 @@ class snprintf(FormatParser):
 
 class __snprintf_chk(FormatParser):
 
-    def run(self, dst_ptr, maxlen, size, fmt):  # pylint:disable=arguments-differ,unused-argument
+    def run(self, dst_ptr, maxlen, flag, size, fmt):  # pylint:disable=arguments-differ,unused-argument
 
         # The format str is at index 4
         fmt_str = self._parse(fmt)
