@@ -54,7 +54,7 @@ class RegionedMemoryMixin(MemoryMixin):
             regioned_memory_cls = RegionedMemory
 
         self._regioned_memory_cls = regioned_memory_cls
-        self._regions: Dict[str,regioned_memory_cls] = { }
+        self._regions: Dict[str, regioned_memory_cls] = { }
 
         self._cle_memory_backer = cle_memory_backer
         self._dict_memory_backer = dict_memory_backer
@@ -67,6 +67,25 @@ class RegionedMemoryMixin(MemoryMixin):
         self._write_targets_limit = write_targets_limit
         self._read_targets_limit = read_targets_limit
 
+    @MemoryMixin.memo
+    def copy(self, memo):
+        o: 'RegionedMemoryMixin' = super().copy(memo)
+        o._write_targets_limit = self._write_targets_limit
+        o._read_targets_limit = self._read_targets_limit
+        o._stack_size = self._stack_size
+        o._endness = self.endness
+        o._stack_region_map = self._stack_region_map
+        o._generic_region_map = self._generic_region_map
+        o._cle_memory_backer = self._cle_memory_backer
+        o._dict_memory_backer = self._dict_memory_backer
+        o._regioned_memory_cls = self._regioned_memory_cls
+
+        o._regions = {}
+        for region_id, region in self._regions.items():
+            o._regions[region_id] = region.copy(memo)
+
+        return o
+
     def load(self, addr, size: Optional[Union[BV,int]]=None, endness=None, condition: Optional[Bool]=None, **kwargs):
 
         if isinstance(size, BV) and isinstance(size._model_vsa, ValueSet):
@@ -133,24 +152,6 @@ class RegionedMemoryMixin(MemoryMixin):
                     r = True
         return r
 
-    @MemoryMixin.memo
-    def copy(self, memo):
-        o: 'RegionedMemoryMixin' = super().copy(memo)
-        o._write_targets_limit = self._write_targets_limit
-        o._read_targets_limit = self._read_targets_limit
-        o._stack_size = self._stack_size
-        o._endness = self.endness
-        o._stack_region_map = self._stack_region_map
-        o._generic_region_map = self._generic_region_map
-        o._cle_memory_backer = self._cle_memory_backer
-        o._dict_memory_backer = self._dict_memory_backer
-        o._regioned_memory_cls = self._regioned_memory_cls
-
-        for region_id, region in self._regions.items():
-            o._regions[region_id] = region.copy(memo)
-
-        return o
-
     def find(self, addr: Union[int,Bits], data, max_search, **kwargs):
         # FIXME: Attempt find() on more than one region
 
@@ -404,8 +405,11 @@ class RegionedMemoryMixin(MemoryMixin):
                 # Convert it to a ValueSet first by annotating it
                 addr_e = addr_e.annotate(RegionAnnotation('global', 0, addr_e._model_vsa))
 
-            for region, offset in addr_e._model_vsa.items():
-                yield region, offset
-
+            model_vsa = addr_e._model_vsa
+            if isinstance(model_vsa, ValueSet):
+                for region, offset in model_vsa.items():
+                    yield region, offset
+            else:
+                raise SimAbstractMemoryError('Cannot parse address as a VSA ValueSet')
         else:
             raise SimAbstractMemoryError('Unsupported address type %s' % type(addr_e))
